/*
 * Copyright 2008 Kyoto University and Honda Motor Co.,Ltd.
 * All rights reserved.
 * HARK was developed by researchers in Okuno Laboratory
 * at the Kyoto University and Honda Research Institute Japan Co.,Ltd.
 */


/*
 * Area.hpp
 *
 *  Created on: 2010/11/18
 *      Author: HARK Development Team
 */

#ifndef DIMPRO_AREA_HPP_
#define DIMPRO_AREA_HPP_

namespace NSDimensionPromise
{

/**
 * Areaの次元定義.
 */
template <typename T>
struct AreaBaseTraits
{
    typedef ScalarNumber<T, 2, 0, 0, 0, 0, 0, 0> ScalarType;
};

/**
 * Areaを表すベースクラス.
 */
template <typename T>
class AreaBase : public AreaBaseTraits<T>::ScalarType
{
public:
    /**
     * Areaを表すスカラー変数を生成する.
     * @param sqmeter 変数の初期値
     */
    explicit AreaBase(const T& sqmeter = T())
        : AreaBaseTraits<T>::ScalarType(sqmeter)
    {  }

    /**
     * Areaを表すスカラー変数をコピーして生成する.
     * @param rhs コピー元の変数
     */
    AreaBase(const typename AreaBaseTraits<T>::ScalarType& rhs)
        : AreaBaseTraits<T>::ScalarType(rhs)
    {  }

    /**
      * 平方メートル単位の値を得る.
      * @return 平方メートル
      */
    const T& ToSquareMeter() const
    {
        return this->X();
    }

    /**
      * 平方ミリメートル単位の値を得る.
      * @return 平方ミリメートル
      */
    const T ToSquareMillimeter() const
    {
        return static_cast<T>(this->X() * 1e6);
    }

    /**
      * 平方メートルからAreaを表すスカラー変数を生成する.
      * @return Areaを表すスカラー変数
      */
    static AreaBase FromSquareMeter(const T& sqmeter)
    {
        return AreaBase(sqmeter);
    }

    /**
      * 平方ミリメートルからAreaを表すスカラー変数を生成する.
      * @return Areaを表すスカラー変数
      */
    static AreaBase FromSquareMillimeter(const T& sqmillimeter)
    {
        return AreaBase(sqmillimeter * 1e-6);
    }
};

/**
 * よく使うAreaの型.
 */
typedef AreaBase<NormalValueType> Area;

}

#endif /* DIMPRO_AREA_HPP_ */
